import { jsx as _jsx } from "react/jsx-runtime";
import { camelToDataKebab, createInternalBlockSpec, createStronglyTypedTiptapNode, getBlockFromPos, getParseRules, inheritedProps, mergeCSSClasses, propsToAttributes, } from "@blocknote/core";
import { NodeViewContent, NodeViewWrapper, ReactNodeViewRenderer, } from "@tiptap/react";
import { renderToDOMSpec } from "./@util/ReactRenderUtil";
// Function that wraps the React component returned from 'blockConfig.render' in
// a `NodeViewWrapper` which also acts as a `blockContent` div. It contains the
// block type and props as HTML attributes.
export function BlockContentWrapper(props) {
    return (
    // Creates `blockContent` element
    _jsx(NodeViewWrapper
    // Adds custom HTML attributes
    , { ...Object.fromEntries(Object.entries(props.domAttributes || {}).filter(([key]) => key !== "class")), 
        // Sets blockContent class
        className: mergeCSSClasses("bn-block-content", props.domAttributes?.class || ""), "data-content-type": props.blockType, ...Object.fromEntries(Object.entries(props.blockProps)
            .filter(([prop, value]) => !inheritedProps.includes(prop) &&
            value !== props.propSchema[prop].default)
            .map(([prop, value]) => {
            return [camelToDataKebab(prop), value];
        })), children: props.children }));
}
// A function to create custom block for API consumers
// we want to hide the tiptap node from API consumers and provide a simpler API surface instead
export function createReactBlockSpec(blockConfig, blockImplementation) {
    const node = createStronglyTypedTiptapNode({
        name: blockConfig.type,
        content: (blockConfig.content === "inline"
            ? "inline*"
            : ""),
        group: "blockContent",
        selectable: true,
        addAttributes() {
            return propsToAttributes(blockConfig.propSchema);
        },
        parseHTML() {
            return getParseRules(blockConfig, blockImplementation.parse);
        },
        renderHTML() {
            // renderHTML is not really used, as we always use a nodeView, and we use toExternalHTML / toInternalHTML for serialization
            // There's an edge case when this gets called nevertheless; before the nodeviews have been mounted
            // this is why we implement it with a temporary placeholder
            const div = document.createElement("div");
            div.setAttribute("data-tmp-placeholder", "true");
            return {
                dom: div,
            };
        },
        addNodeView() {
            return (props) => ReactNodeViewRenderer((props) => {
                // Gets the BlockNote editor instance
                const editor = this.options.editor;
                // Gets the block
                const block = getBlockFromPos(props.getPos, editor, this.editor, blockConfig.type);
                // Gets the custom HTML attributes for `blockContent` nodes
                const blockContentDOMAttributes = this.options.domAttributes?.blockContent || {};
                // hacky, should export `useReactNodeView` from tiptap to get access to ref
                const ref = NodeViewContent({}).ref;
                const BlockContent = blockImplementation.render;
                return (_jsx(BlockContentWrapper, { blockType: block.type, blockProps: block.props, propSchema: blockConfig.propSchema, domAttributes: blockContentDOMAttributes, children: _jsx(BlockContent, { block: block, editor: editor, contentRef: ref }) }));
            }, {
                className: "bn-react-node-view-renderer",
            })(props);
        },
    });
    return createInternalBlockSpec(blockConfig, {
        node: node,
        toInternalHTML: (block, editor) => {
            const blockContentDOMAttributes = node.options.domAttributes?.blockContent || {};
            const BlockContent = blockImplementation.render;
            const output = renderToDOMSpec((refCB) => (_jsx(BlockContentWrapper, { blockType: block.type, blockProps: block.props, propSchema: blockConfig.propSchema, domAttributes: blockContentDOMAttributes, children: _jsx(BlockContent, { block: block, editor: editor, contentRef: refCB }) })), editor);
            output.contentDOM?.setAttribute("data-editable", "");
            return output;
        },
        toExternalHTML: (block, editor) => {
            const blockContentDOMAttributes = node.options.domAttributes?.blockContent || {};
            const BlockContent = blockImplementation.toExternalHTML || blockImplementation.render;
            const output = renderToDOMSpec((refCB) => {
                return (_jsx(BlockContentWrapper, { blockType: block.type, blockProps: block.props, propSchema: blockConfig.propSchema, domAttributes: blockContentDOMAttributes, children: _jsx(BlockContent, { block: block, editor: editor, contentRef: refCB }) }));
            }, editor);
            output.contentDOM?.setAttribute("data-editable", "");
            return output;
        },
    });
}
//# sourceMappingURL=ReactBlockSpec.js.map